local RayUI, E, _, V, P, G = unpack(select(2, ...))
local LSM = E.LSM

local _G = _G
local ipairs, strsplit, strsub = ipairs, strsplit, strsub
local GetAddOnEnableState = GetAddOnEnableState

RayUI.dummy = function() return end
RayUI.Title = "|cff7aa6d6Ray|r|cffff0000U|r|cff7aa6d6I|r "
RayUI.Version = GetAddOnMetadata("ElvUI_RayUI", "Version")
RayUI.ElvUIV = tonumber(E.version)
RayUI.ElvUIX = tonumber(GetAddOnMetadata("ElvUI_RayUI", "X-ElvVersion"))
RayUI.WoWPatch, RayUI.WoWBuild, RayUI.WoWPatchReleaseDate, RayUI.TocVersion = GetBuildInfo()
RayUI.WoWBuild = select(2, GetBuildInfo()) RayUI.WoWBuild = tonumber(RayUI.WoWBuild)

function RayUI:cOption(name)
	local color = "|cff7aa6d6%s |r"
	return (color):format(name)
end

function RayUI:SetMoverPosition(mover, point, anchor, secondaryPoint, x, y)
	if not _G[mover] then return end
	local frame = _G[mover]

	frame:ClearAllPoints()
	frame:SetPoint(point, anchor, secondaryPoint, x, y)
	E:SaveMoverPosition(mover)
end

function RayUI:RegisterDB(tbl, path)
	if (not RayUI["RegisteredDBs"]) then
		RayUI["RegisteredDBs"] = {}
	end
	self:UpdateRegisteredDB(tbl, path)
	RayUI["RegisteredDBs"][tbl] = path
end

function RayUI:UpdateRegisteredDB(tbl, path)
	local path_parts = {strsplit(".", path)}
	local _db = E.db.mui
	for _, path_part in ipairs(path_parts) do
		_db = _db[path_part]
	end
	tbl.db = _db
end

function RayUI:UpdateRegisteredDBs()
	if (not RayUI["RegisteredDBs"]) then
		return
	end

	local dbs = RayUI["RegisteredDBs"]

	for tbl, path in pairs(dbs) do
		self:UpdateRegisteredDB(tbl, path)
	end
end

function RayUI:OnProfileChanged()
	RayUI:Hook(E, "UpdateEnd", "UpdateAll")
end

function RayUI:UpdateAll()
	self:UpdateRegisteredDBs()
	for _, module in ipairs(self:GetRegisteredModules()) do
		local mod = RayUI:GetModule(module)
		if (mod and mod.ForUpdateAll) then
			mod:ForUpdateAll()
		end
	end
	self:Unhook(E, "UpdateEnd")
end

function RayUI:UpdateRegisteredDB(tbl, path)
	local path_parts = {strsplit(".", path)}
	local _db = E.db.rayui
	for _, path_part in ipairs(path_parts) do
		_db = _db[path_part]
	end
	tbl.db = _db
end

function RayUI:RegisterDB(tbl, path)
	if (not RayUI["RegisteredDBs"]) then
		RayUI["RegisteredDBs"] = {}
	end
	self:UpdateRegisteredDB(tbl, path)
	RayUI["RegisteredDBs"][tbl] = path
end

function RayUI:IsAddOnEnabled(addon) -- Credit: Azilroka
	return GetAddOnEnableState(E.myname, addon) == 2
end

-- Disable profanityFilter.  By EKE
function RayUI:PLAYER_LOGIN(event)
	self:UnregisterEvent(event)

	if GetCVar("portal") ~= "CN" then return end

	ConsoleExec("portal TW")
	SetCVar("profanityFilter", 0)
end

function E:ShortenString(str, numChars, dots)
	local bytes = #str
	if bytes <= numChars then
		return str
	else
		local len, pos = 0, 1
		while pos <= bytes do
			len = len + 1
			local c = str:byte(pos)
			if c > 0 and c <= 127 then
				pos = pos + 1
			elseif c >= 192 and c <= 223 then
				pos = pos + 2
			elseif c >= 224 and c <= 239 then
				len = len + 1
				pos = pos + 3
			elseif c >= 240 and c <= 247 then
				len = len + 1
				pos = pos + 4
			end
			if len >= numChars then
				break
			end
		end

		if len >= numChars and pos <= bytes then
			return strsub(str, 1, pos - 1)..(dots and '...' or '')
		else
			return str
		end
	end
end
